# Note: this is not used anymore.

import boto3
import sys
from pprint import pprint

import config
import util

from util import debug

_cfg = config.Config()

# class method
def list():
    debug("In cluster.py class method list")
    clusters = _cfg.get('clusters')
    cluster_names = clusters.keys()
    cluster_names.sort()
    print "Defined clusters:"
    for cluster_name in cluster_names:
        cluster = Cluster(cluster_name)
        status = cluster.status()
        print "    {}:".format(cluster_name)
        for k,v in status.items():
            print "        {}: {}".format(k, v)

def vpc_conn():
    debug("In cluster.py class method vpc_conn")
    ec2 = boto3.resource('ec2')
    vpc_iterator = ec2.vpcs.all()
    return vpc_iterator

def get_vpc_by_name(name):
    debug("In cluster.py class method get_vpc_by_name")
    # TODO: error if more than one vpc has the same Name tag
    vpcs_iterator = vpc_conn() # iterator of all vpcs

    ret = None
    for vpc in vpcs_iterator:
        tags = vpc.tags
        for tag in tags:
             if tag['Key'] == "Name":
                 if tag['Value'] == name:
                     ret = vpc
    return ret

class Cluster:
    def __init__(self, name):    
        if name not in _cfg.get('clusters'):
            print "Invalid cluster name: {}".format(name)
            return
        self.cluster_config = _cfg.get('clusters', name)
        self.name = name
        self.conn = vpc_conn()

    def status(self):
        debug("In cluster.py status")
        vpc = get_vpc_by_name(self.name)
        if not vpc:
            return { 'exists': False }

        # check if this VPC is managed by udo
        tags = vpc.tags
        vpc_is_managed = False
        if 'Udo' in tags:
            vpc_is_managed = True

        return {
            'udo': vpc_is_managed,
            'exists': True,
            'state': vpc.state,
            'id': vpc.id
        }
    
    # active this cluster
    #
    # NOTE: this might not work right
    # We might get rid of cluster.py completely
    def activate(self):
        debug("In cluster.py activate")
        conn = vpc_conn()

        cfg = _cfg.get('clusters', self.name)
        if not cfg:
            print "No configuration found for {}".format(self.name)
            return False

        vpc = get_vpc_by_name(self.name)
        if vpc:
            print "Cluster {} already exists".format(self.name)
            return

        # create VPC
        #
        # boto3 docs example for creating a vpc:
        # ec2 = boto3.resource('ec2')
        # vpc = ec2.create_vpc(CidrBlock='10.0.0.0/24')
        # subnet = vpc.create_subnet(CidrBlock='10.0.0.0/25')
        # gateway = ec2.create_internet_gateway()
 
        subnet_cidr = cfg.get('subnet_cidr')
        if not subnet_cidr:
            print "No subnet definition found for {}".format(self.name)
            return False
        vpc = conn.create_vpc(subnet_cidr)
        vpc.add_tag('Name', value=self.name)
        # for now assume that our subnet is the same CIDR as the VPC
        # this is simpler but less fancy
        subnet = conn.create_subnet(vpc.id, subnet_cidr)
        # all done
        util.message_integrations("Created VPC {}".format(self.name))

        # mark that this cluster is being managed by udo
        vpc.add_tag('udo', value=True)

        return True



import boto3
import sys
import time
from pprint import pprint

import config
import util
import launchconfig
import cluster

from time import sleep
from util import debug

_cfg = config.Config()

class AutoscaleGroup:
    def __init__(self, cluster_name, role_name):
        self.cluster_name = cluster_name
        self.role_name = role_name
        self.role_config = config.get_role_config(cluster_name, role_name)
        self.conn = util.as_conn()

    def has_valid_role(self):
        if not self.role_config:
            return False
        return True

    def name(self):
        debug("In asgroup.py name")
        return "-".join([self.cluster_name, self.role_name])

    def exists(self):
        debug("In asgroup.py exists")
        asg = self.get_asgroup()
        if asg:
            return asg
        return False

    def find_vpc_subnet_by_cidr(self, cidr):
        debug("In asgroup.py find_vpc_subnet_by_cidr")
        vpc = cluster.get_vpc_by_name(self.cluster_name)
        if not vpc:
            print "Failed to find VPC {}".format(self.cluster_name)
            return None
        # look for subnet that matches
        vpc_iterator = cluster.vpc_conn()
        vpc = None
        for _vpc in vpc_iterator:
          if _vpc.cidr_block == cidr:
              vpc = _vpc
        if not vpc:
            pprint("unable to find vpc with cidr " + str(cidr))
            return None
        return vpc

    # returns LaunchConfig for this ASgroup
    # may or may not exist
    def lc(self):
        debug("In asgroup.py lc")
        lc = launchconfig.LaunchConfig(self.cluster_name, self.role_name)
        if self.exists():
            asgroup = self.get_asgroup()
            if not asgroup:
                # NOTE:
                # this might be a race condition between 
                # exists() and get_asgroup()
                return None

            blc = None
            if 'LaunchConfigurationName' in asgroup:
                blc = asgroup['LaunchConfigurationName']
            if blc and 'LaunchConfigurationName' in blc:
                lc.set_name(blc)
        return lc

    # returns true if the LC exists
    def activate_lc(self):
        debug("In asgroup.py activate_lc")
        # make sure we have a launchconfig activated
        lc = self.lc()
        if not lc:
            # might need to wait a min
            util.retry(lambda: self.lc(), 60)
            lc = self.lc()
        if not lc:
            print "Timed out waiting to create LaunchConfiguration"
            return false
        if lc.exists():
            print "Using LaunchConfig {}".format(lc.get_lc_server_name())
            return True
        print "Creating LaunchConfig {}".format(lc.name())
        return lc.activate()

    # we can't modify a launchconfig in place, we have to create
    # a new one and set that as our lc
    def update_lc(self):
        debug("In asgroup.py update_lc")
        oldlc = self.lc()
        oldname = oldlc.get_lc_server_name()

        lc = oldlc.update() # create new version

        asgroup = self.get_asgroup()
        asg_name = asgroup['AutoScalingGroupName']

        lcname = lc.get_lc_server_name()
        
        client = util.as_conn()
        debug("updating asg: " + asg_name + " with LaunchConfig " + lcname)

        response = client.update_auto_scaling_group( AutoScalingGroupName = asg_name, LaunchConfigurationName = lcname )

        # delete old
        conn = util.as_conn()
        if oldname is not lcname:
            # TODO: waiter
            debug("deleting Launchconfig " + oldname)
            conn.delete_launch_configuration(LaunchConfigurationName = oldname)

    def get_asgroup(self):
        debug("In asgroup.py get_asgroup")
        conn = util.as_conn()
        asg = conn.describe_auto_scaling_groups( AutoScalingGroupNames = [ self.name() ] )
        if not asg['AutoScalingGroups']:
            return None
        else:
            asg = asg['AutoScalingGroups'][0]
        return asg

    def get_num_instances(self):
        ag = util.as_conn()
        asg_info = ag.describe_auto_scaling_groups( AutoScalingGroupNames = [ self.name() ] )
        return len(asg_info['AutoScalingGroups'][0]['Instances'])

    # get desired_size
    def get_scale_size(self):
        debug("In asgroup.py get_scale_size")
        asgroup = self.get_asgroup()
        print "Desired: {}\nMin:{}\nMax:{}".format(asgroup['DesiredCapacity'], asgroup['MinSize'], asgroup['MaxSize'])

    # set desired_size
    def scale(self, desired):
        debug("In asgroup.y scale")
        asgroup = self.get_asgroup()

        if desired < asgroup['MinSize']:
            print "Cannot scale: {} is lower than min_size ({})".format(desired, asgroup['MinSize'])
            return
        if desired > asgroup['MaxSize']:
            print "Cannot scale: {} is greater than max_size ({})".format(desired, asgroup['MaxSize'])
            if not util.confirm("Increase max_size to {}?".format(desired)):
                return
            asgroup['MaxSize'] = desired
        current = asgroup['DesiredCapacity']

        # Set DesiredCapacity
        response = util.as_conn().set_desired_capacity( AutoScalingGroupName = self.name(), DesiredCapacity = desired )

        # Check if DesiredCapacity was changed
        debug("in asgroup.py scale: running 'asgroup = self.get_asgroup()'")
        asgroup = self.get_asgroup()
        new = asgroup['DesiredCapacity']
        if (new != current):
            msg = "Changed ASgroup {} desired_capacity from {} to {}".format(self.name(), current, new)
            util.message_integrations(msg)

    # kill off asg and recreate it
    def reload(self):
        # skip deactivation if it doesn't exist
        asgroup = self.get_asgroup()
        if not asgroup or not self.exists():
            self.activate()
            return

        debug("In asgroup.py reload")
        if not util.confirm("Are you sure you want to tear down the {} ASgroup and recreate it?".format(self.name())):
            return
        util.message_integrations("Reloading ASgroup {}".format(self.name()))
        self.deactivate()
        util.retry(lambda: self.activate(), 60)

    def deactivate(self): # a.k.a asg destroy
        # NOTE
        # deleting asg logic should be in its own function

        # * delete ASG by reducing capacities of asg to 0
        # * delete launchconfig
        #
        # reducing ASG capacities to 0 triggers eventual instance
        # termination
        debug("In asgroup.py deactivate")        

        asg_name = self.name()
        ag = util.as_conn()
        ec2 = util.ec2_conn()
    
        asg_info = ag.describe_auto_scaling_groups( AutoScalingGroupNames = [ asg_name ] )

        if not asg_info['AutoScalingGroups']:
            print("ASG does not exist.  Maybe it was already deleted?")
        else:
            # delete the ASG
            num_instances = len(asg_info['AutoScalingGroups'][0]['Instances'])
            if self.get_num_instances() == 0:
                pprint("There are no instances in asg: " + asg_name)
                print("Deleting asg: " + asg_name)
                response = ag.delete_auto_scaling_group( AutoScalingGroupName=asg_name )
                util.message_integrations("Deleted ASgroup {}".format(asg_name))
            else:
                debug("There are " + str(num_instances) + " instances that need to be removed from asg: " + asg_name)
                debug("terminating instances in asg: " + asg_name)
                debug("by setting to 0 MinSize, MaxSize, DesiredCapacity")
                response = ag.update_auto_scaling_group(AutoScalingGroupName = asg_name, MinSize=0, MaxSize=0, DesiredCapacity=0)
                debug("Waiting 30 seconds to give AWS time to terminate the instances")

                if self.get_num_instances() != 0:
                    util.retry(lambda: ag.delete_auto_scaling_group(AutoScalingGroupName=asg_name), 300)
                if self.get_num_instances() != 0 or self.get_num_instances():
                    print("unable to delete instances in asg.")
                    return False
                util.message_integrations("Deleted ASgroup {}".format(asg_name))

        # if launch config exists, delete it 
        lc = self.lc()
        if not lc.exists():
            print("launchconfig does not exist.  Maybe you deleted it already?")
        else:
            lc.deactivate()
        return True

    def get_subnet_ids_by_cidrs(self, cidrs):
        debug("In asgroup.y get_subnet_ids_by_cidrs")
        ret = []
        _ec2 = util.ec2_conn()
        subnets = _ec2.describe_subnets()['Subnets']
        for cidr in cidrs:
            for subnet in subnets:
                if subnet['CidrBlock'] == cidr:
                    ret.append(subnet['SubnetId'])
        return ret

    # NOTE: Need to test with multiple availability zones and multiple subnet ids
    #
    # creates the LaunchConfig
    def activate(self):
        debug("In asgroup.py activate")
        conn = util.as_conn()
        cfg = self.role_config
        name = self.name()
        if not self.activate_lc(): # ensure this LaunchConfig already exists
            return False

        subnet_cidrs = cfg.get('subnets_cidr')
        if not subnet_cidrs or not len(subnet_cidrs) or None in subnet_cidrs:
            print "Valid subnets_cidr are required for {}/{}".format(self.cluster_name, self.role_name)
            return False
        print("Using subnets " + str(subnet_cidrs))

        subnet_ids = self.get_subnet_ids_by_cidrs(subnet_cidrs)
        azs=cfg.get('availability_zones')
        cfg_args = {}

        # If AvailabilityZones is defined, add it to the args we will pass to conn.create_auto_scaling_group()
        if azs:
            cfg_args['AvailabilityZones'] = azs
            print "AZs: {}".format(azs)
        else:
            pprint("No availability_zones set")

        # VPCZoneIdentifier ( which can be plural ) takes a string
        subnet_ids_string=''
        _length = len(subnet_ids)
        for subnet_id in subnet_ids:
            subnet_ids_string=subnet_ids_string + subnet_id
            if _length > 1:
                subnet_ids_string=subnet_ids_string + ', '
            _length = _length - 1
        pprint("Using subnet ids: " + str(subnet_ids_string))

        cfg_args['AutoScalingGroupName'] = self.name()
        cfg_args['DesiredCapacity'] = cfg.get('scale_policy')['desired']
        cfg_args['LoadBalancerNames'] = cfg.get('elbs')
        cfg_args['LaunchConfigurationName'] = self.lc().get_lc_server_name()
        cfg_args['MaxSize'] = cfg.get('scale_policy', 'max_size')
        cfg_args['MinSize'] = cfg.get('scale_policy', 'min_size')
        cfg_args['VPCZoneIdentifier'] = subnet_ids_string

        if not cfg_args['LoadBalancerNames']:
            cfg_args['LoadBalancerNames'] = []

        response = conn.create_auto_scaling_group(**cfg_args)
        # NOTE: should check if asg was created

        debug('Preparing tags that will be applied to the asg')
        tags = cfg.get('tags')
        if not tags:
            tags = {}
        tags['cluster'] = self.cluster_name
        tags['role'] = self.role_name
        tags['Name'] = self.name()

        # apply tags        
        tag_set = [self.ag_tag(name, k,v) for (k,v) in tags.iteritems()]
        debug("Applying tags to asg")
        conn.create_or_update_tags(Tags=tag_set)

        util.message_integrations("Activated ASgroup {}".format(name))
        # NOTE: what should we be returning here?  Not sure.
        #return ag
        return name

    def ag_tag(self, ag, k, v):
        debug("In asgroup.py ag_tag")
        return { 'ResourceId': ag,
             'ResourceType': 'auto-scaling-group', # ResourceType must be string 'auto-scaling-group'
             'Key': str(k), # Key must be a string
             'Value': str(v), # Value must be a string
             'PropagateAtLaunch': True,
        }

    # NOTE: not sure what this is for. not sure what the token stuff is for
    def get_instances(self):
        debug("In asgroup.py get_instances")
        ret = []
        done = False
        next_token = None
        while not done:
            params = {}
            if next_token:
                params["next_token"] = next_token
            all_instances = self.conn.get_all_autoscaling_instances(**params)
            next_token = all_instances.next_token
            if not next_token:
                done = True
            instances = [i for i in all_instances if i.group_name == self.name()]
            ret.extend(instances)
        return ret

    def print_instances(self):
        debug("In asgroup.py print_instances")
        name = self.name()
        asg_info = self.conn.describe_auto_scaling_groups( AutoScalingGroupNames = [name] )['AutoScalingGroups']
        if not asg_info:
            pprint("No info for ASG " + name + " . are you sure it exists?")
            return None
        else:
            instances = asg_info[0]['Instances']
            print("Group\t\tID\t\tState\t\tStatus")
            for instance in instances:
                status = instance['HealthStatus']
                lcname = instance['LaunchConfigurationName']
                state = instance['LifecycleState']
                group = name
                iid = instance['InstanceId']
                print("{}\t{}\t{}\t{}".format(group, iid, state, status))

    def ip_addresses(self):
        debug("In asgroup.py ip_addresses")
        instances = self.conn.describe_auto_scaling_groups( AutoScalingGroupNames = [self.name()] )['AutoScalingGroups'][0]['Instances']
        ids = [i['InstanceId'] for i in instances]
        ec2_conn = util.ec2_conn()
        ec2_instances = ec2_conn.describe_instances( InstanceIds = ids )['Reservations']
        ips = [ instance['Instances'][0]['PublicIpAddress'] for instance in ec2_instances ]
        return ips



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse 
import os
import random
import sys
import warnings

from pprint import pprint

import asgroup
import config
import deploy
import launchconfig
import util

# top-level commands go here
class Udo:
    # launchconfig
    def lc(self, *args):
        args = list(args)
        if not len(args) or not args[0]:
            print "launchconfig command requires an action. Valid actions are: "
            print " cloudinit (cluster).(role) - view cloud_init bootstrap script"
            print " create (cluster).(role) - create launch configuration"
            print " destroy (cluster).(role) - delete launch configuration"
            return
        action = args.pop(0)

        cluster,role,extra = self.get_cluster_and_role_from_args(*args)

        if not cluster or not role:
            return

        lc = launchconfig.LaunchConfig(cluster, role)

        if action == 'cloudinit':
            cloudinit = lc.cloud_init_script()
            print cloudinit
        elif action == 'create':
            lc.activate()
        elif action == 'destroy':
            lc.deactivate()
        else:
            print "Unrecognized LaunchConfig action"

    # autoscale
    def asg(self, *args):
        args = list(args)
        if not len(args) or not args[0]:
            print "asgroup command requires an action. Valid actions are: "
            print " instances (cluster).(role) - list instances in group"
            print " randomip (cluster).(role) - get an IP address of a host in the group"
            print " create (cluster).(role) - create an autoscale group"
            print " destroy (cluster).(role) - delete an autoscale group and terminate all instances"
            print " reload (cluster).(role) - destroys asgroup and launchconfig, then recreates them"
            print " updatelc (cluster).(role) - generates a new launchconfig version"
            print " scale (cluster).(role) - view current scaling settings"
            print " scale (cluster).(role) (desired) - set desired number of instances"
            return
        action = args.pop(0)

        # TODO: hook up 'list'

        cluster, role, extra = self.get_cluster_and_role_from_args(*args)
        if not cluster or not role:
            return

        ag = asgroup.AutoscaleGroup(cluster, role)

        if not ag.has_valid_role():
            print "Invalid role {} specified for cluster {}".format(role, cluster)
            return

        if action == 'create':
            ag.activate()
        elif action == 'destroy':
            ag.deactivate()
        elif action == 'reload':
            ag.reload()
        elif action == 'updatelc':
            ag.update_lc()
        elif action == 'instances':
            ag.print_instances()
        elif action == 'randomip':
            ips = ag.ip_addresses()
            print(random.choice(ips))
        elif action == 'scale':
            # get scale arg
            if extra:
                scale = int(extra)
                print("in main.py: ag.scale(scale)")
                ag.scale(scale)
            else:
                print("in main.py: ag.get_scale_size()")
                ag.get_scale_size()
        else:
            print "Unrecognized asgroup action {}".format(action)

    # CodeDeploy
    def deploy(self, *args):
        args = list(args)
        if not len(args) or not args[0]:
            print "deploy command requires an action. Valid actions are: "
            print " list applications"
            print " list groups [application]"
            print " list deployments"
            print " list configs"
            print " create (group) (commit_id)"
            print " last [group]"
            print " status deploymentId" # only for debugging
            return
        action = args.pop(0)

        if action == 'list':
            dep = deploy.Deploy()
            if not len(args):
                print "list what? applications, groups, deployments, post or configs?"
                return
            what = args.pop(0)
            if what == 'applications' or what == 'apps':
                dep.list_applications()
            elif what == 'groups':
                # application name?
                application = None
                if len(args):
                    application = args.pop(0)
                dep.list_groups(application)
            elif what == 'configs':
                dep.list_configs()
            elif what == 'deployments':
                dep.list_deployments()
            elif what == 'post':
                dep.list_post_deploy_hooks()
            else:
                print "Unknown list type: {}".format(what)
        elif action == 'create':
            # require group, commit_id
            if len(args) != 2:
                print "deploy create requires group and commit id"
                return
            group = args.pop(0)
            commit_id = args.pop(0)
            dep = deploy.Deploy()
            dep.create(group, commit_id)
        elif action == 'last':
            dep = deploy.Deploy()
            group_name = None
            if len(args) == 1:
                group_name = args.pop(0)
            dep.print_last_deployment(deployment_group_name=group_name)
        elif action == 'status':
            deploymentId = args.pop(0)
            dep = deploy.Deploy()
            print(dep.deployment_status(deploymentId))
        elif action == 'stop':
            dep = deploy.Deploy()
            group_name = None
            if len(args) == 1:
                group_name = args.pop(0)
            dep.stop_deployment(deployment_group_name=group_name)
        elif len(args) == 1:
            # assume we want to create a deployment
            group = action
            commit_id = args.pop(0)
            dep = deploy.Deploy()
            dep.create(group, commit_id)
        else:
            print "Unknown deploy command: {}".format(action)

    def version(self, *args):
        args = list(args)
        print('3.0.2')

    # for testing features
    def test(self, *args):
        args = list(args)
        if not len(args) or not args[0]:
            print "test command requires an action. Valid actions are: "
            print " integrations"
            return
        action = args.pop(0)

        if action == 'integrations':
            util.message_integrations("Testing Udo integrations")
        else:
            print "Unknown test command: {}".format(action)

    # returns cluster_name,role_name,rest_of_args
    def get_cluster_and_role_from_args(self, *args):
        args = list(args)
        # need cluster/role
        if len(args) < 1:
            print "Please specify cluster.role target for this command"
            return None,None,None
        cluster_role = args.pop(0)

        help = "Please specify the target of your action using the format: cluster.role"

        cluster_name = None
        role_name = None
        if '.' in cluster_role:
            # split on .
            cluster_name, role_name = cluster_role.split(".")
            if not cluster_name:
                print("Cluster name not specified.", help)
            if not role_name:
                print("Role name not specified.", help)
        else:
            # assume we're just talking about a cluster with one role
            cluster_name = cluster_role

        cluster = config.get_cluster_config(cluster_name)
        if not cluster:
            print "Unknown cluster {}".format(cluster_name)
            return None,None,None

        roles = cluster.get('roles')
        if not roles:
            print "Invalid configuration for {}: no roles are defined".format(cluster_name)
            return None,None,None

        if not role_name:
            role_names = roles.keys()
            if len(role_names) == 1:
                # assume the only role
                print "No role specified, assuming {}".format(role_names[0])
                role_name = role_names[0]
            else:
                print "Multiple roles available for cluster {}".format(cluster_name)
                for r in role_names:
                    print "  - {}".format(r)
                return None,None,None

        if not role_name in roles:
            print("Role {} not found in cluster {} configuration".format(role_name, cluster_name))
            return None,None,None

        # still stuff?
        extra = None
        if len(args):
            extra = args.pop(0)

        return cluster_name, role_name, extra

def invoke_console():
    # argument parsing
    parser = argparse.ArgumentParser(description='Manage AWS clusters.')
    parser.add_argument('cmd', metavar='command', type=str, nargs='?',
        help='Action to perform. Valid actions: status.')
    parser.add_argument('cmd_args', metavar='args', type=str, nargs='*',
        help='Additional arguments for command.')
    args = parser.parse_args()
    
    if args.cmd not in dir(Udo):
        if args.cmd:
            print "'{}' is not a valid command".format(args.cmd)
        else:
            print "You must specify a command"
        # full command summary
        print """
Valid commands are:
  * lc cloudinit - display cloud-init script
  * lc create - create a launch configuration
  * lc destroy - delete a launch configuration
  * asg instances - print instances in autoscaling groups
  * asg reload - destroy and create an autoscaling group to update the config
  * asg create - create an autoscaling group
  * asg destroy - delete an autoscaling group
  * asg updatelc - updates launchconfiguration in-place
  * asg scale - set desired number of instances
  * asg randomip - print IP address of random host in group
  * deploy list apps - view CodeDeploy applications
  * deploy list groups - view CodeDeploy application deployment groups
  * deploy list deployments - view CodeDeploy deployment statuses
  * deploy list configs - view CodeDeploy configurations
  * deploy list post - view post deploy hooks
  * deploy [create] (group) (commit) - create new deployment for commit on group
  * deploy last - shows status of most recent deployment
  * deploy stop - cancel last deployment
  * version - print udo version
        """
        sys.exit(1)

    # execute cmd
    exe = Udo()
    method = getattr(exe, args.cmd)
    method(*args.cmd_args)

if __name__ == '__main__':
    invoke_console()



# Handy common utilities for udo modules

import boto3
import getpass
import json
import os
import socket
import sys
import urllib2
import botocore

from pprint import pprint
from time import sleep

import config

# don't use for tests

def debug(msg):
    # to use:
    # export DEBUG='True'
    if os.environ.get('DEBUG') == 'True':
        pprint(msg)
    
def default_config():
    return config.Config()

def region():
    region = default_config().get('region')
    return region

# returns dict of arguments common to any boto connection we establish
def connection_args():
    debug("in util.py connection_args")
    _region = region()
    if not _region:
        # use the default region, I guess?
        print("Warning: default region is not configured")
        return "us-east-1"

    args = {
        'region_name': _region
    }
    return args

# AutoScaling connection
def as_conn():
    debug("in util.py as_conn")
    args = connection_args()
    return boto3.client('autoscaling', **args)

# EC2 connection
def ec2_conn():
    debug("in util.py ec2_conn")
    args = connection_args()
    return boto3.client('ec2', **args)

# CodeDeploy connection
def deploy_conn():
    debug("in util.py deploy_conn")
    args = connection_args()
    return boto3.client('codedeploy', **args)

# ask a yes/no question
# returns true/false
def confirm(msg):
    debug("in util.py confirm")
    yn = raw_input(msg + " (y/n) ")
    if yn.lower() == 'y':
        return True
    print "Aborted"
    return False

# keep trying proc until timeout or no exception is thrown
# use this when you're waiting for a change to take effect
# FIXME: actually respect timeout
# "waiters" are better but they are not implemented for everything we need (yet)
def retry(proc, timeout):
    debug("in util.py retry")
    success = False
    ret = None
    while success == False:
        try:
            ret = proc()     
            success = True
        except botocore.exceptions.ClientError as e:
            if default_config().get('debug'):
                # dump response
                print "Error: {}, retrying...".format(e)
            else:
                print('.'),
                sys.stdout.flush()
                sleep(5)
    print "\n"
    return ret

def user_and_host():
    debug("in util.py user_and_host")
    username = getpass.getuser()
    hostname = socket.gethostname()
    return "{}@{}".format(username, hostname)

# also prints out msg
def message_integrations(msg):
    debug("in util.py message_integrations")
    message_slack("[{}]  {}".format(user_and_host(), msg))
    print msg

def message_slack(msg):
    debug("in util.py message_slack")
    slack_cfg = default_config().new_root('slack')
    if not slack_cfg:
        # not configured
        return

    slack_url      = slack_cfg.get('url')
    slack_username = slack_cfg.get('username')
    slack_channel  = slack_cfg.get('channel')

    payload = {
        'username': slack_username,
        'text': msg,
        'channel': slack_channel,
    }

    data = json.dumps(payload)
    headers = {'Content-Type': 'application/json'}
    request = urllib2.Request(slack_url, data, headers=headers)
    return urllib2.urlopen(request).read()
